@@ -25,15 +25,14 @@ from .helper import split_authors
 from .constants import BookMeta
 
 
-
-def extractCover(zipFile, coverFile, coverpath, tmp_file_name):
-    if coverFile is None:
+def extract_cover(zip_file, cover_file, cover_path, tmp_file_name):
+    if cover_file is None:
         return None
     else:
-        zipCoverPath = os.path.join(coverpath, coverFile).replace('\\', '/')
-        cf = zipFile.read(zipCoverPath)
+        zip_cover_path = os.path.join(cover_path, cover_file).replace('\\', '/')
+        cf = zip_file.read(zip_cover_path)
         prefix = os.path.splitext(tmp_file_name)[0]
-        tmp_cover_name = prefix + '.' + os.path.basename(zipCoverPath)
+        tmp_cover_name = prefix + '.' + os.path.basename(zip_cover_path)
         image = open(tmp_cover_name, 'wb')
         image.write(cf)
         image.close()
@@ -47,12 +46,12 @@ def get_epub_info(tmp_file_path, original_file_name, original_file_extension):
         'dc': 'http://purl.org/dc/elements/1.1/'
     }
 
-    epubZip = zipfile.ZipFile(tmp_file_path)
+    epub_zip = zipfile.ZipFile(tmp_file_path)
 
-    txt = epubZip.read('META-INF/container.xml')
+    txt = epub_zip.read('META-INF/container.xml')
     tree = etree.fromstring(txt)
     cfname = tree.xpath('n:rootfiles/n:rootfile/@full-path', namespaces=ns)[0]
-    cf = epubZip.read(cfname)
+    cf = epub_zip.read(cfname)
     tree = etree.fromstring(cf)
 
     coverpath = os.path.dirname(cfname)
@@ -86,9 +85,9 @@ def get_epub_info(tmp_file_path, original_file_name, original_file_extension):
     lang = epub_metadata['language'].split('-', 1)[0].lower()
     epub_metadata['language'] = isoLanguages.get_lang3(lang)
 
-    epub_metadata = parse_epbub_series(ns, tree, epub_metadata)
+    epub_metadata = parse_epub_series(ns, tree, epub_metadata)
 
-    coverfile = parse_ebpub_cover(ns, tree, epubZip, coverpath, tmp_file_path)
+    cover_file = parse_epub_cover(ns, tree, epub_zip, coverpath, tmp_file_path)
 
     if not epub_metadata['title']:
         title = original_file_name
@@ -100,7 +99,7 @@ def get_epub_info(tmp_file_path, original_file_name, original_file_extension):
         extension=original_file_extension,
         title=title.encode('utf-8').decode('utf-8'),
         author=epub_metadata['creator'].encode('utf-8').decode('utf-8'),
-        cover=coverfile,
+        cover=cover_file,
         description=epub_metadata['description'],
         tags=epub_metadata['subject'].encode('utf-8').decode('utf-8'),
         series=epub_metadata['series'].encode('utf-8').decode('utf-8'),
@@ -108,37 +107,43 @@ def get_epub_info(tmp_file_path, original_file_name, original_file_extension):
         languages=epub_metadata['language'],
         publisher="")
 
-def parse_ebpub_cover(ns, tree, epubZip, coverpath, tmp_file_path):
-    coversection = tree.xpath("/pkg:package/pkg:manifest/pkg:item[@id='cover-image']/@href", namespaces=ns)
-    coverfile = None
-    if len(coversection) > 0:
-        coverfile = extractCover(epubZip, coversection[0], coverpath, tmp_file_path)
+
+def parse_epub_cover(ns, tree, epub_zip, cover_path, tmp_file_path):
+    cover_section = tree.xpath("/pkg:package/pkg:manifest/pkg:item[@id='cover-image']/@href", namespaces=ns)
+    cover_file = None
+    if len(cover_section) > 0:
+        cover_file = extract_cover(epub_zip, cover_section[0], cover_path, tmp_file_path)
     else:
         meta_cover = tree.xpath("/pkg:package/pkg:metadata/pkg:meta[@name='cover']/@content", namespaces=ns)
         if len(meta_cover) > 0:
-            coversection = tree.xpath("/pkg:package/pkg:manifest/pkg:item[@id='"+meta_cover[0]+"']/@href", namespaces=ns)
+            cover_section = tree.xpath(
+                "/pkg:package/pkg:manifest/pkg:item[@id='"+meta_cover[0]+"']/@href", namespaces=ns)
+            if not cover_section:
+                cover_section = tree.xpath(
+                    "/pkg:package/pkg:manifest/pkg:item[@properties='" + meta_cover[0] + "']/@href", namespaces=ns)
         else:
-            coversection = tree.xpath("/pkg:package/pkg:guide/pkg:reference/@href", namespaces=ns)
-        if len(coversection) > 0:
-            filetype = coversection[0].rsplit('.', 1)[-1]
+            cover_section = tree.xpath("/pkg:package/pkg:guide/pkg:reference/@href", namespaces=ns)
+        if len(cover_section) > 0:
+            filetype = cover_section[0].rsplit('.', 1)[-1]
             if filetype == "xhtml" or filetype == "html":  # if cover is (x)html format
-                markup = epubZip.read(os.path.join(coverpath, coversection[0]))
-                markupTree = etree.fromstring(markup)
+                markup = epub_zip.read(os.path.join(cover_path, cover_section[0]))
+                markup_tree = etree.fromstring(markup)
                 # no matter xhtml or html with no namespace
-                imgsrc = markupTree.xpath("//*[local-name() = 'img']/@src")
+                img_src = markup_tree.xpath("//*[local-name() = 'img']/@src")
                 # Alternative image source
-                if not len(imgsrc):
-                    imgsrc = markupTree.xpath("//attribute::*[contains(local-name(), 'href')]")
-                if len(imgsrc):
-                    # imgsrc maybe startwith "../"" so fullpath join then relpath to cwd
-                    filename = os.path.relpath(os.path.join(os.path.dirname(os.path.join(coverpath, coversection[0])),
-                                                            imgsrc[0]))
-                    coverfile = extractCover(epubZip, filename, "", tmp_file_path)
+                if not len(img_src):
+                    img_src = markup_tree.xpath("//attribute::*[contains(local-name(), 'href')]")
+                if len(img_src):
+                    # img_src maybe start with "../"" so fullpath join then relpath to cwd
+                    filename = os.path.relpath(os.path.join(os.path.dirname(os.path.join(cover_path, cover_section[0])),
+                                                            img_src[0]))
+                    cover_file = extract_cover(epub_zip, filename, "", tmp_file_path)
             else:
-                coverfile = extractCover(epubZip, coversection[0], coverpath, tmp_file_path)
-    return coverfile
+                cover_file = extract_cover(epub_zip, cover_section[0], cover_path, tmp_file_path)
+    return cover_file
 
-def parse_epbub_series(ns, tree, epub_metadata):
+
+def parse_epub_series(ns, tree, epub_metadata):
     series = tree.xpath("/pkg:package/pkg:metadata/pkg:meta[@name='calibre:series']/@content", namespaces=ns)
     if len(series) > 0:
         epub_metadata['series'] = series[0]
