@@ -59,10 +59,10 @@ $("#archived_cb").on("change", function() {
         )
     };
 
-    $("#shelf-actions").on("click", "[data-shelf-action]", function (e) {
+    $("#add-to-shelves, #remove-from-shelves").on("click", "[data-shelf-action]", function (e) {
         e.preventDefault();
         $.ajax({
-                url: this.href,
+                url: $(this).data('href'),
                 method:"post",
                 data: {csrf_token:$("input[name='csrf_token']").val()},
             })
@@ -72,7 +72,7 @@ $("#archived_cb").on("change", function() {
                     case "add":
                         $("#remove-from-shelves").append(
                             templates.remove({
-                                add: this.href,
+                                add: $this.data('href'),
                                 remove: $this.data("remove-href"),
                                 content: $("<div>").text(this.textContent).html()
                             })
@@ -82,7 +82,7 @@ $("#archived_cb").on("change", function() {
                         $("#add-to-shelves").append(
                             templates.add({
                                 add: $this.data("add-href"),
-                                remove: this.href,
+                                remove: $this.data('href'),
                                 content: $("<div>").text(this.textContent).html(),
                             })
                         );
