@@ -20,11 +20,12 @@ from datetime import datetime
 from cps.services.worker import CalibreTask, STAT_FINISH_SUCCESS
 
 class TaskUpload(CalibreTask):
-    def __init__(self, taskMessage):
+    def __init__(self, taskMessage, book_title):
         super(TaskUpload, self).__init__(taskMessage)
         self.start_time = self.end_time = datetime.now()
         self.stat = STAT_FINISH_SUCCESS
         self.progress = 1
+        self.book_title = book_title
 
     def run(self, worker_thread):
         """Upload task doesn't have anything to do, it's simply a way to add information to the task list"""
@@ -34,4 +35,4 @@ class TaskUpload(CalibreTask):
         return "Upload"
 
     def __str__(self):
-        return "Upload {}".format(self.message)
+        return "Upload {}".format(self.book_title)
